// ExecutionPatternDlg.cpp : implementation file
//

#include "stdafx.h"

#include <stdexcept>
#include <stack>
#include <string>
#include <cassert>

#include "Common.h"
#include "StrategyOrderPlacing.h"
#include "ExecutionPatternDlg.h"

namespace
{
   CString TypeToString(eExecutionPatternParameterType e)
   {
      switch (e)
      {
      case epptBool:
         return _T("Bool");
      case epptInt32:
         return _T("Int32");
      case epptDouble:
         return _T("Double");
      case epptString:
         return _T("String");
      case epptObject:
         return _T("Object");
      case epptCollection:
         return _T("Collection");
      case epptCollectionItem:
         return _T("CollectionItem");
      default:
         assert(!"Undefined parameter type");
      }
      return _T("");
   }

   CString GetNameFromItemText(CString& itemText)
   {
      int i = itemText.Find(_T(':'));
      if(i > 0)
      {
         return itemText.Left(i-1);
      }
      return _T("");
   }

   typedef CExecutionPatternDlg::PatternParameterPtr PatternParameterPtr; 
   typedef CExecutionPatternDlg::PatternParametersPtr PatternParametersPtr;
}


// ExecutionPatternDlg dialog
IMPLEMENT_DYNAMIC(CExecutionPatternDlg, CDialog)
CExecutionPatternDlg::CExecutionPatternDlg(ATL::CComPtr<ICQGCEL>& spCel, 
                                           ATL::CComPtr<ICQGExecutionPattern>& spPattern,
                                           const CString& patternName,
                                           std::map<CString, ATL::CComPtr<ICQGExecutionPattern> >& patterns,
                                           ATL::CComPtr<ICQGStrategyDefinition> spDef,
                                           CWnd* pParent /*=NULL*/)
	: m_spCQGCEL(spCel)
   , m_spPattern(spPattern)
   , m_patternName(patternName)
   , m_patterns(patterns)
   , m_spDefinition(spDef)
   , m_isNewPattern(patternName.IsEmpty())
   , CDialog(CExecutionPatternDlg::IDD, pParent)
{
   if(!m_isNewPattern)
   {
      m_spPattern = ClonePattern(spPattern);
   }
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

CExecutionPatternDlg::~CExecutionPatternDlg()
{
}

ATL::CComPtr<ICQGExecutionPattern> CExecutionPatternDlg::GetPattern() const
{
   return m_spPattern;
}

const CString& CExecutionPatternDlg::GetPatternName() const
{
   return m_patternName;
}

BOOL CExecutionPatternDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      // Set the icon for this dialog.  The framework does this automatically
      // when the application's main window is not a dialog
      SetIcon(m_hIcon, TRUE);       // Set big icon
      SetIcon(m_hIcon, FALSE);      // Set small icon

      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CExecutionPatternDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_PATTERNS_TREE, m_trPatternView);
   DDX_Control(pDX, IDC_EDPATTERN_NAME, m_edPatternName);
   DDX_Control(pDX, IDC_CMBPATTERN_TYPE, m_cmbPatternType);
   DDX_Control(pDX, IDC_BUTTON_CREATE, m_btnCreate);
   DDX_Control(pDX, IDC_CHDEFAULT_PATTERN, m_chDefaultPattern);
   DDX_Control(pDX, IDC_EDPARAM_NAME, m_edParamName);
   DDX_Control(pDX, IDC_BUTTON_ADD, m_btnParamAdd);
   DDX_Control(pDX, IDC_CMBPARAM_TYPE, m_cmbParamType);
   DDX_Control(pDX, IDC_BUTTON_PARAM_REMOVE_ALL, m_btnParamRemoveAll);
   DDX_Control(pDX, IDC_BUTTON_PARAM_REMOVE, m_btnParamRemove);
   DDX_Control(pDX, IDC_EDPARAM_VALUE, m_edParamValue);
   DDX_Control(pDX, IDOK, m_btnOk);
}

BEGIN_MESSAGE_MAP(CExecutionPatternDlg, CDialog)
   ON_BN_CLICKED(IDC_BUTTON_CREATE, OnBnClickedButtonCreate)
   ON_BN_CLICKED(IDC_CHDEFAULT_PATTERN, OnBnClickedChdefaultPattern)
   ON_EN_CHANGE(IDC_EDPATTERN_NAME, OnEnChangeEdpatternName)
   ON_BN_CLICKED(IDC_BUTTON_ADD, OnBnClickedButtonAdd)
   ON_EN_CHANGE(IDC_EDPARAM_NAME, OnEnChangeEdparamName)
   ON_NOTIFY(TVN_SELCHANGED, IDC_PATTERNS_TREE, OnTvnSelchangedPatternsTree)
   ON_BN_CLICKED(IDOK, OnBnClickedOk)
   ON_BN_CLICKED(IDC_BUTTON_PARAM_REMOVE, OnBnClickedButtonParamRemove)
   ON_BN_CLICKED(IDC_BUTTON_PARAM_REMOVE_ALL, OnBnClickedButtonParamRemoveAll)
   ON_CBN_SELCHANGE(IDC_CMBPARAM_TYPE, OnCbnSelchangeCmbparamType)
END_MESSAGE_MAP()


void CExecutionPatternDlg::InitializeControls()
{
   // Initialize pattern related controls
   // Initialize execution pattern default types combobox
   int cur = m_cmbPatternType.AddString(_T("Limit"));
   m_cmbPatternType.SetItemData(cur, otLimit);
   cur = m_cmbPatternType.AddString(_T("Market"));
   m_cmbPatternType.SetItemData(cur, otMarket);
   cur = m_cmbPatternType.AddString(_T("Stop"));
   m_cmbPatternType.SetItemData(cur, otStop);
   m_cmbPatternType.SetCurSel(0);

   // Initialize controls and show pattern which is selected to modify
   if(!m_isNewPattern)
   {
      m_edPatternName.SetWindowText(m_patternName);
      HTREEITEM item = m_trPatternView.InsertItem(m_patternName);
      m_trPatternView.SelectItem(item);

      PatternParametersPtr spPatternsParams;
      GET_PARAM(m_spPattern, spPatternsParams, get_Parameters);
 
      ShowPatternParams(spPatternsParams, item);
   }
   
   // Initialize parameter types combo
   cur = m_cmbParamType.AddString(_T("Bool"));
   m_cmbParamType.SetItemData(cur, epptBool);
   cur = m_cmbParamType.AddString(_T("Int32"));
   m_cmbParamType.SetItemData(cur, epptInt32);
   cur = m_cmbParamType.AddString(_T("Double"));
   m_cmbParamType.SetItemData(cur, epptDouble);
   cur = m_cmbParamType.AddString(_T("String"));
   m_cmbParamType.SetItemData(cur, epptString);
   cur = m_cmbParamType.AddString(_T("Object"));
   m_cmbParamType.SetItemData(cur, epptObject);
   cur = m_cmbParamType.AddString(_T("Collection"));
   m_cmbParamType.SetItemData(cur, epptCollection);
   cur = m_cmbParamType.AddString(_T("CollectionItem"));
   m_cmbParamType.SetItemData(cur, epptCollectionItem);
   m_cmbParamType.SetCurSel(0);

   EnableControls();
}

void CExecutionPatternDlg::ShowPatternParams(PatternParametersPtr spPatternParams,
                                             HTREEITEM parentItem)
{
   long count;
   GET_PARAM(spPatternParams, count, get_Count);

   for (long index = 0; index < count; ++index)
   {
      PatternParameterPtr spParam;
      HRESULT hr = spPatternParams->get_Item(CComVariant(index), &spParam);
      AssertCOMError(spPatternParams, hr);
      AddParamToView(spParam, parentItem);
   }
   m_edParamName.SetFocus();
   m_edParamName.SetSel(0, -1);

   EnableControls();
}


void CExecutionPatternDlg::AddParamToView(PatternParameterPtr& spParam, 
                                          HTREEITEM parentItem)
{
   eExecutionPatternParameterType type;
   GET_PARAM(spParam, type, get_Type);

   ATL::CComBSTR name;
   GET_PARAM(spParam, name, get_Name);

   ATL::CComBSTR value;
   GET_PARAM(spParam, value, get_Value);

   CString treeItemName = ATL::CW2T(name) + _T(" : ") + TypeToString(type) + _T(" : ") + ATL::CW2T(value);
   
   HTREEITEM currentItem = AddItemToTree(treeItemName, parentItem);
   
   if(type == epptCollection || type == epptObject || type == epptCollectionItem)
   {
      PatternParametersPtr spPatternsParams;
      GET_PARAM(spParam, spPatternsParams, get_Parameters);

      ShowPatternParams(spPatternsParams, currentItem);
   }
}

HTREEITEM CExecutionPatternDlg::AddItemToTree( const CString& itemText, HTREEITEM parentItem )
{
   HTREEITEM item = m_trPatternView.InsertItem(itemText, parentItem);
   m_trPatternView.Expand(parentItem, TVE_EXPAND);
   m_trPatternView.Select(m_trPatternView.GetSelectedItem(), TVGN_CARET);
   return item;
}

// ExecutionPatternDlg message handlers

void CExecutionPatternDlg::OnBnClickedButtonCreate()
{
   CString name;
   m_edPatternName.GetWindowText(name);

   // Check pattern for existence
   if(m_patterns.find(name) != m_patterns.end())
   {
      AfxMessageBox(_T("Pattern with same name already exists"), MB_ICONSTOP);
      return;
   }

   try
   {
      // Add pattern root to view
      AddItemToTree(name, m_trPatternView.GetRootItem());

      // Creation of empty execution pattern
      if(m_chDefaultPattern.GetCheck() != BST_CHECKED)
      {
         HRESULT hr = m_spCQGCEL->CreateExecutionPattern(NULL, otUndefined, &m_spPattern);
         AssertCOMError(m_spCQGCEL, hr);
      }
      // Creation of default execution pattern
      else if(m_chDefaultPattern.GetCheck() == BST_CHECKED)
      {
         assert(m_spDefinition != NULL);

         eOrderType type;
         type = static_cast<eOrderType>(m_cmbPatternType.GetItemData(m_cmbPatternType.GetCurSel()));
         HRESULT hr = m_spCQGCEL->CreateExecutionPattern(m_spDefinition, type, &m_spPattern);
         AssertCOMError(m_spCQGCEL, hr);

         PatternParametersPtr spPatternsParams;
         GET_PARAM(m_spPattern, spPatternsParams, get_Parameters);

         // Show newly crated pattern
         HTREEITEM parentItem = m_trPatternView.GetRootItem();
         ShowPatternParams(spPatternsParams, parentItem);
      }
      
      // Selects root item
      m_trPatternView.SelectItem(m_trPatternView.GetRootItem());

      // Holds new created pattern name
      m_patternName = name;

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExecutionPatternDlg::OnBnClickedChdefaultPattern()
{
    m_cmbPatternType.EnableWindow(m_chDefaultPattern.GetCheck()== BST_CHECKED);
}

void CExecutionPatternDlg::OnEnChangeEdpatternName()
{
   EnableControls();
}

void CExecutionPatternDlg::OnBnClickedButtonAdd()
{
   try
   {
      // Recognition of parameters collection where will be added new parameter
      PatternParametersPtr spPatternsParams;

      // Search selected parent parameter to add nested
      std::stack<PatternParameterPtr > path;
      m_spCurrentParam = FindParamPath(m_trPatternView.GetSelectedItem(), path);

      if(m_spCurrentParam == NULL)
      {
         GET_PARAM(m_spPattern, spPatternsParams, get_Parameters);
      }
      else
      {
         GET_PARAM(m_spCurrentParam, spPatternsParams, get_Parameters);
      }

      // Get parameter name, type and value
      CString name;
      m_edParamName.GetWindowText(name);
      assert(!name.IsEmpty());

      eExecutionPatternParameterType type;
      int typeIndex = m_cmbParamType.GetCurSel();
      assert(typeIndex != CB_ERR);
      type = static_cast<eExecutionPatternParameterType>(m_cmbParamType.GetItemData(typeIndex));

      CString value;
      m_edParamValue.GetWindowText(value);

      // Search param with same name in collection
      PatternParameterPtr spParam;
      HRESULT hr = spPatternsParams->get_Item(CComVariant(name), &spParam);
      AssertCOMError(spPatternsParams, hr);

      if(spParam == NULL)
      {
         // Add param to collection
         PatternParameterPtr spPatternsParam;
         hr = spPatternsParams->Add(ATL::CT2W(name), type, ATL::CT2W(value), &spParam);
         AssertCOMError(spPatternsParams, hr);

         AddParamToView(spParam, m_trPatternView.GetSelectedItem());
      }
      else
      {
         // Param couldn't be added to this collection
         throw std::invalid_argument("Parameter with same name already exists");
      }

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExecutionPatternDlg::OnEnChangeEdparamName()
{
   EnableControls();
}

/// \brief Search given parameter in execution pattern
/// \param spParams Start point collection to search
/// \param names Stack of ancestor's names
/// \param path Input Stack to fill ancestor parameters
PatternParameterPtr FindByName(const PatternParametersPtr& spParams,
                               std::stack<CString>& names,
                               std::stack<PatternParameterPtr >& path )
{
   // if name empty return 
   if(names.empty())
   {
      return CExecutionPatternDlg::PatternParameterPtr();   
   }

   // Search in collection
   PatternParameterPtr spParam;
   HRESULT hr = spParams->get_Item(CComVariant(names.top()), &spParam);
   AssertCOMError(spParams, hr);

   // Couldn't find first ancestor
   if(spParam == NULL)
   {
      return spParam;
   }

   // Collect ancestors
   path.push(spParam);

   eExecutionPatternParameterType type;
   GET_PARAM(spParam, type, get_Type);

   // Removes from ancestors collection and return parameter if collection get empty
   names.pop();
   if(names.empty())
   {
      // Current parameter is a leaf node (simple parameter) or
      // non-leaf node is selected
      return spParam;
   }

   // Found non simple parameter, getting its parameters collection for future search recursive call
   ASSERT(type == epptObject || type == epptCollection  || type == epptCollectionItem);
   PatternParametersPtr spSubParams;
   GET_PARAM(spParam, spSubParams, get_Parameters);
   
   return FindByName(spSubParams, names, path);
}

PatternParameterPtr CExecutionPatternDlg::FindParamPath(const HTREEITEM& item,
                                                        std::stack<PatternParameterPtr >& path ) const
{
   HTREEITEM hItem = item;
   // Ancestors names collection
   std::stack<CString> names; 
   
   // Walking up in pattern tree and collect ancestor's names while root item not reached 
   while(true)
   {
      CString itemText = m_trPatternView.GetItemText(hItem);
      hItem = m_trPatternView.GetParentItem(hItem);
      if(hItem == NULL)
      {
         break;
      }
      names.push(GetNameFromItemText(itemText));
   }

   // Search parameter in pattern params based on collected ancestor's names
   if(names.size() > 0)
   {
      PatternParametersPtr spParams;
      GET_PARAM(m_spPattern, spParams, get_Parameters);

      return FindByName(spParams, names, path);
   }
   
   // Couldn't find param
   return PatternParameterPtr();
}

namespace
{
   void CopyParams(const ATL::CComPtr<ICQGExecutionPatternParameters>& spParamsFrom,
                        ATL::CComPtr<ICQGExecutionPatternParameters>& spParamsTo)
   {
      long count;
      GET_PARAM(spParamsFrom, count, get_Count);

      for (long i  = 0; i < count; ++i)
      { 
         ATL::CComPtr<ICQGExecutionPatternParameter> spParameter;
         HRESULT hr = spParamsFrom->get_Item(CComVariant(i), &spParameter);
         AssertCOMError(spParamsFrom, hr);

         ATL::CComBSTR name;
         GET_PARAM(spParameter, name, get_Name);

         eExecutionPatternParameterType type;
         GET_PARAM(spParameter, type, get_Type);

         ATL::CComBSTR value;
         GET_PARAM(spParameter, value, get_Value);  

         ATL::CComPtr<ICQGExecutionPatternParameter> spNewParameter;
         spParamsTo->Add(name, type, value, &spNewParameter);

         if(type == epptCollection || type == epptObject || type == epptCollectionItem)
         {
            ATL::CComPtr<ICQGExecutionPatternParameters> spSubParamsFrom;
            GET_PARAM(spParameter, spSubParamsFrom, get_Parameters);

            ATL::CComPtr<ICQGExecutionPatternParameters> spSubParamsTo;
            GET_PARAM(spNewParameter, spSubParamsTo, get_Parameters);

            CopyParams(spSubParamsFrom, spSubParamsTo);
         }
      }   
   }
}

ATL::CComPtr<ICQGExecutionPattern> CExecutionPatternDlg::ClonePattern(
                                          const ATL::CComPtr<ICQGExecutionPattern>& spPattern) const
{
   ATL::CComPtr<ICQGExecutionPattern> spNewPattern;
   HRESULT hr = m_spCQGCEL->CreateExecutionPattern(NULL, otUndefined, &spNewPattern);
   ATLVERIFY(SUCCEEDED(hr));

   ATL::CComPtr<ICQGExecutionPatternParameters> spParams;
   GET_PARAM(spPattern, spParams, get_Parameters);

   ATL::CComPtr<ICQGExecutionPatternParameters> spNewParams;
   GET_PARAM(spNewPattern, spNewParams, get_Parameters);
   
   CopyParams(spParams, spNewParams);

   return spNewPattern;
}


void CExecutionPatternDlg::OnTvnSelchangedPatternsTree(NMHDR * /*pNMHDR*/, LRESULT *pResult)
{
   EnableControls();
   
   *pResult = 0;
}


void CExecutionPatternDlg::OnBnClickedOk()
{
   // Inserting pattern into main patterns collection
   if(m_spPattern && m_isNewPattern)
   {
      m_patterns[m_patternName] = m_spPattern;
   }

   OnOK();
}

void CExecutionPatternDlg::OnBnClickedButtonParamRemove()
{
   try
   {
      // Recognition of parameters collection from which will be removed parameter
      std::stack<PatternParameterPtr > path;
      m_spCurrentParam = FindParamPath(m_trPatternView.GetSelectedItem(), path);

      PatternParametersPtr spParams;
      ATL::CComBSTR name;
      HRESULT hr;

      switch(path.size())
      {
         // Selected root must remove pattern
      case 0:
         assert(!"Can't remove pattern");
         break;
         // Selected item under root, must be empty pattern
      case 1:
         GET_PARAM(m_spCurrentParam, name, get_Name);
         GET_PARAM(m_spPattern, spParams, get_Parameters);

         // Remove parameter
         hr = spParams->Remove(CComVariant(name));
         AssertCOMError(spParams, hr);

         // Redraw pattern after remove param
         RedrawTree();
         break;
         // Remove selected parameter
      default:
         assert(path.top() == m_spCurrentParam);
         GET_PARAM(m_spCurrentParam, name, get_Name);

         path.pop();

         PatternParametersPtr spParams;
         GET_PARAM(path.top(), spParams, get_Parameters);

         // Remove parameter
         hr = spParams->Remove(CComVariant(name));
         AssertCOMError(spParams, hr);

         // Redraw pattern after remove param
         RedrawTree();
         break;
      }
      m_trPatternView.SelectItem(m_trPatternView.GetRootItem());
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExecutionPatternDlg::RedrawTree()
{
   try
   {
      // Remove all items from pattern view
      m_trPatternView.DeleteItem(m_trPatternView.GetRootItem());
      HTREEITEM item = m_trPatternView.InsertItem(m_patternName);
      m_trPatternView.SelectItem(item);

      PatternParametersPtr spPatternsParams;
      GET_PARAM(m_spPattern, spPatternsParams, get_Parameters);

      // Add items to view
      ShowPatternParams(spPatternsParams, m_trPatternView.GetRootItem());   
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExecutionPatternDlg::OnBnClickedButtonParamRemoveAll()
{
   try
   {
      PatternParametersPtr spPatternsParams;
      GET_PARAM(m_spPattern, spPatternsParams, get_Parameters);

      long count;
      GET_PARAM(spPatternsParams, count, get_Count);

      // Remove all parameters under root
      for (long i = 0; i < count; ++i)
      {
         HRESULT hr = spPatternsParams->Remove(ATL::CComVariant(0));
         AssertCOMError(spPatternsParams, hr);
      }

      m_spCurrentParam.Release();

      // Redraw pattern
      RedrawTree();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CExecutionPatternDlg::EnableControls()
{
   // Initialize enable/disable flags
   BOOL isPatternNameExists = m_edPatternName.LineLength() > 0;
   BOOL isPatternExists = m_spPattern != NULL;
   BOOL hasParam = m_trPatternView.GetCount() > 1;
   BOOL isRootSelected = m_trPatternView.GetSelectedItem() != m_trPatternView.GetRootItem();

   eOrderType type;
   type = static_cast<eOrderType>(m_cmbParamType.GetItemData(m_cmbParamType.GetCurSel()));
   BOOL collectionType = (type == epptCollection || type == epptObject || type == epptCollectionItem);
   
   // Initialize pattern related controls
   m_btnCreate.EnableWindow(m_isNewPattern && isPatternNameExists && !isPatternExists);
   m_edPatternName.EnableWindow(m_isNewPattern && !isPatternExists);
   m_chDefaultPattern.EnableWindow(!isPatternExists && (m_spDefinition != 0));
   m_cmbPatternType.EnableWindow(m_chDefaultPattern.GetCheck()== BST_CHECKED && !isPatternExists);

   BOOL isParamNameExists = m_edParamName.LineLength() > 0;
   // Initialize parameter related controls
   m_edParamName.EnableWindow(isPatternExists);
   m_cmbParamType.EnableWindow(isPatternExists && isParamNameExists);
   m_edParamValue.EnableWindow(isPatternExists && isParamNameExists && !collectionType);
   m_btnParamAdd.EnableWindow((isPatternExists) && isParamNameExists);
   m_btnParamRemove.EnableWindow(isPatternExists && isRootSelected && hasParam);
   m_btnParamRemoveAll.EnableWindow(isPatternExists && hasParam);

   m_btnOk.EnableWindow(isPatternExists);
}

void CExecutionPatternDlg::OnCbnSelchangeCmbparamType()
{
   m_edParamValue.SetWindowText(_T(""));
   EnableControls();
}
