#pragma once
#include "afxcmn.h"


class CTransactionDlg;

/**
\typedef ATL::IDispEventImpl<1, CTransactionDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImplTransaction
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CTransactionDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                              >
                              ICQGCELDispEventImplTransaction;

// CTransactionDlg dialog

class CTransactionDlg : public CDialog, public ICQGCELDispEventImplTransaction
{
	DECLARE_DYNAMIC(CTransactionDlg)

public:
   /// \brief Constructor
   /// \param spCel ICQGCEL instance
   /// \param spOrder Order instance to show transactions
   /// \param pParent Parent window
   CTransactionDlg(const ATL::CComPtr<ICQGCEL>& spCEL,
                   ATL::CComPtr<ICQGOrder> spOrder,
                   CWnd* pParent = NULL);

	virtual ~CTransactionDlg();

   // Dialog Data
	enum { IDD = IDD_TRANSACTION_DIALOG };

public:
   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CTransactionDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 17, OnOrderChanged)
   END_SINK_MAP()

protected:
   /// \brief Fired when new order was added, order status was 
   /// \brief changed or the order was removed.
   /// \param change - The type of the occurred change
   /// \param order - CQGOrder object representing the order to which the change refers
   /// \param oldProperties - CQGOrderProperties collection representing 
   /// \param the old values of the changed order properties
   /// \param fill - CQGFill object representing the last fill of the order
   /// \param cqgerr - CQGError object to describe the last error, if any
   /// \return S_OK
   STDMETHOD(OnOrderChanged)(
                              eChangeType change, 
                              ICQGOrder* order, 
                              ICQGOrderProperties* oldProperties, 
                              ICQGFill* fill, 
                              ICQGError* cqgerr);


protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Dialog icon.
   HICON m_hIcon;

public:
   /// \brief Process selection change in transaction list view
   afx_msg void OnLvnItemchangedLsvTransactions(NMHDR *pNMHDR, LRESULT *pResult);

   /// \brief Prepare dialog to destroy
   afx_msg void OnDestroy();

private:
   /// \brief Shows specified order's all transactions in the list control.
   /// \param spOrder - ICQGOrder instance
   void ShowTransactions(const ATL::CComPtr<ICQGOrder>& spOrder);

   /// \brief Shows specified transaction data to the list control.
   /// \param spTransaction - ICQGTransaction instance
   void ShowTransaction(const ATL::CComPtr<ICQGTransaction>& spTransaction);

   /// \brief Shows transaction's hang orders ids in the list control.
   /// \param spTransaction - ICQGTransaction instance
   void ShowHangOrderIds(const ATL::CComPtr<ICQGTransaction>& spTransaction);

   /// \brief Clears transaction's hang order ids from the list control.
   void ClearHangOrderIds();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes transactions list control.
   void InitializeTransactionsList();

   /// \brief Initializes transaction's hang order ids list control.
   void InitializeHangOrderIdsList();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \return String representation of the value
   CString GetValueAsString(ATL::CComVariant value, VARENUM valueType = VT_EMPTY) const;

   /// \brief Returns string representation of the specified order event.
   /// \param event - order event
   /// \return String representation of the order event
   static CString OrderEventToString(eOrderEvent event);


	DECLARE_MESSAGE_MAP()
private:
   /// \brief Transactions list view
   CListCtrl m_lvOrderTransactions;
   
   /// \brief Hang ids list view
   CListCtrl m_lvHangOrderIds;

private:
   /// \brief Not available value.
   static const CString m_N_A;

   /// \brief Order holder
   ATL::CComPtr<ICQGOrder> m_spOrder;

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

private:
   /// \enum eOrderTransactionColumn. Represents order transaction columns in the list.
   enum eOrderTransactionColumn;

   /// \enum eHangOrderIdsColumn. Represents transaction's hang orders ids columns in the list.
   enum eHangOrderIdsColumn;
};

enum CTransactionDlg::eOrderTransactionColumn
{
   otcTimestamp,
   otcTransactionID,
   otcStatus,
   otcFillID,
   otcRejectReason,
   otcPropName,
   otcPropPreviousValue,
   otcPropCurrentValue
};

enum CTransactionDlg::eHangOrderIdsColumn
{
   oidcHangOrderId
};